/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.ilesteban.rulenotbeingfired;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.KnowledgePackage;
import org.drools.event.rule.ActivationCancelledEvent;
import org.drools.event.rule.ActivationCreatedEvent;
import org.drools.event.rule.DefaultAgendaEventListener;
import org.drools.io.Resource;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.junit.Test;

/**
 *
 * @author esteban
 */
public class TestA {
    
    @Test
    public void doTest(){
        
        Map<Resource, ResourceType> resources = new LinkedHashMap<Resource, ResourceType>();
        resources.put(ResourceFactory.newClassPathResource("rules/rules.drl"), ResourceType.DRL);
        
        KnowledgeBase kbase = this.createKbase(this.compileResources(resources));
        
        StatefulKnowledgeSession ksession = this.createKsession(kbase);
        
        
        
        DataSample data = new DataSample();
        //data.addValue(Parameter.PARAM_A, 10.0);
        
        ksession.insert(data);
        
        ksession.fireAllRules();
    }

    private StatefulKnowledgeSession createKsession(KnowledgeBase kbase){
        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
        
        ksession.addEventListener(new DefaultAgendaEventListener(){

            @Override
            public void activationCreated(ActivationCreatedEvent event) {
                System.out.println("Activation created for "+event.getActivation().getRule().getName());
            }
            
            @Override
            public void activationCancelled(ActivationCancelledEvent event) {
                System.out.println("Activation cancelled for "+event.getActivation().getRule().getName());
            }
            
        });
        
        return ksession;
    }
    
    private KnowledgeBase createKbase(Collection<KnowledgePackage> knowledgePackages){
        KnowledgeBaseConfiguration conf = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
       // conf.setOption( EventProcessingOption.STREAM );

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase(conf);
        kbase.addKnowledgePackages(knowledgePackages);

        return kbase;
    }
    
    private Collection<KnowledgePackage> compileResources(Map<Resource, ResourceType> resources) {

        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

        for (Map.Entry<Resource, ResourceType> entry : resources.entrySet()) {
            kbuilder.add(entry.getKey(), entry.getValue());
            if (kbuilder.hasErrors()) {
                Logger.getLogger(TestA.class.getName()).log(Level.SEVERE, "Compilation Errors in {0}", entry.getKey());
                Iterator<KnowledgeBuilderError> iterator = kbuilder.getErrors().iterator();
                while (iterator.hasNext()) {
                    KnowledgeBuilderError knowledgeBuilderError = iterator.next();
                    Logger.getLogger(TestA.class.getName()).log(Level.SEVERE, knowledgeBuilderError.getMessage());
                    System.out.println(knowledgeBuilderError.getMessage());
                }
                throw new IllegalStateException("Compilation Errors");
            }
        }

        return kbuilder.getKnowledgePackages();
    }
    
}
